pub enum PveError {
UrlParse(ParseError),
Reqwest(Error),
Unauthorized(String),
NotFound(String),
Client(String),
Server(String),
Unknown(String),
Serialize(Error),
Deserialize(Error),
}Expand description
Used as a very broad error type encompassing serialization, request construction, parsing URLs, and response status codes. If you want to actually understand why something failed, you’d probably want to use a match statement.
Variants
UrlParse(ParseError)
Reqwest(Error)
NotFound(String)
Client(String)
Server(String)
Unknown(String)
Serialize(Error)
Deserialize(Error)